<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: MaintainBin.php 4338 2006-04-14 22:41:39Z wenzel $

/**
  * MaintainBin.php - contains the {@link MaintainBin} class
  *
  * @author $Author$
  * @filesource
  * @package    bin
  */

require_once dirname(__FILE__).'/includes.php';

/**
  * Interface for command-line tools in Maintain.  This class contains methods
  * for parsing command-line input, variables for parameter configs, etc.
  *
  * @package    bin
  */
class MaintainBin {

// {{{ constructor

    /**
      * Constructor.  Runs setup() to load global config ($conf) and
      * optionally sets the command-line arguments array
      *
      * @param array    command-line arguments array
      {@link http://pear.php.net/package/Console_Getargs/docs/latest/Console_Getargs/Console_Getargs.html#methodfactory}
      */
    function MaintainBin($args = array())
    {
        setup();

        $this->setConfig($args);
    }

// }}}

// {{{ vars

    /**
      * Configuration array for command-line handling
      *
      * @link {http://pear.php.net/package/Console_Getargs/docs/latest/Console_Getargs/Console_Getargs.html#methodfactory}
      */
    var $_config;

    /**
      * The parsed arguments object (returned from Console_Getargs::factory())
      *
      * @link {http://pear.php.net/package/Console_Getargs/docs/latest/Console_Getargs/Console_Getargs.html#methodfactory}
      */
    var $args;

// }}}
// {{{ setConfig()

    /**
      * Set the array of valid command-line arguments
      *
      * @link {http://pear.php.net/package/Console_Getargs/docs/latest/Console_Getargs/Console_Getargs.html#methodfactory}
      * @param  array   config array
      */
    function setConfig($config)
    {
        $this->_config = $config;
    }

// }}}
// {{{ parseArgs()

    /**
      * Parse the command-line input against the config array
      *
      * @return object  PEAR_Console_Getargs_Options object on success, 
                        exits with usage string on fail
      */
    function parseArgs()
    {
        $this->args = Console_Getargs::factory($this->_config);

        if (!PEAR::isError($this->args)) {
            return $this->args;
        }

        // queue up a string exit with a failure
        $failure = '';

        switch ($this->args->getCode()) {

        case CONSOLE_GETARGS_ERROR_USER:
            $failure = Console_Getargs::getHelp($this->_config, 
                    $this->args->getMessage() . "\n") . "\n";
            break;

        case CONSOLE_GETARGS_HELP:
            $failure = Console_Getargs::getHelp($this->_config) . "\n";
            break;

        default:
            $failure = _("Unknown error")."\n";

        }

        $this->fail($failure);
    }

// }}}

// {{{ confirm
    /**
      * Confirm an action with a yes/no dialog.
      * Used to confirm forced/irreversible actions.
      *
      * @param  String  yes/no question to be displayed to the user
      * @return Boolean true if confirmed with yes, false otherwise
      */
    function confirm($msg)
    {
        if (!isset($msg)) $msg = _("Do you really want to do this?");

        echo $msg;
        echo "\n\n"._("Please confirm with (y)es or (n)o: ");
        
        // read info from standard input
        $decision = fgetc(STDIN);
        
        // compare with internationalized first yes/no character and return that
        return (strtolower(substr($decision, 0, 1)) == strtolower(substr(_("yes"), 0, 1)));
    }
// }}}

// {{{ fail()

    /**
      * Terminate the script with ERROR status
      *
      * @param  mixed   string error message or PEAR_Error
      * @return void
      */
    function fail($msg = '')
    {
        if (PEAR::isError($msg)) {
            $msg = $msg->getMessage();
        }

        $this->finish($msg, ERROR);
    }

// }}}
// {{{ finish()

    /**
      * Terminate the script with given status, displaying given text first
      *
      * @param  string  message to display before terminating
      * @param  int     error code to terminate the script with
      * @return void
      */
    function finish($msg = '', $code = SUCCESS)
    {
        echo $msg . "\n";
        exit($code);
    }

// }}}

}

?>
