<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/dao/extended/dhcpoption.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: dhcpoption.php 4175 2006-02-08 21:33:18Z bartel $

/**
  * adminoption.php - contains the {@link DHCPOption} class
  *
  * @author $Author: bartel $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

require_once DAODIR.'/Dhcp_options.php';

/**
  * This class works with the dhcp_option table to manage
  * dhcp options inside Maintain
  *
  * @package    classes
  * @subpackage dao
  */
class DHCPOption extends DAO_Dhcp_options {

    // {{{ constructor

    /**
      * Constructor
      *
      * @param  int ID (primary key) of this object in the dhcp_options table
      */
	function DHCPOption ($option_id=0) 
    {
        if ($option_id) {
            $this->id = $option_id;
            $this->get($this->id);
        }

	} // DHCPOption

    // }}}

/* Validators */
    // {{{ validateName()

    /**
      * Validate the name field
      *
      * @return mixed   true if valid, error string if false
      */
	function validateName() 
    {
        // make sure a name has been set
        if (!$this->name) {
            return _("Must specify a name for this option");
        }

        // make sure this is a unique name
        $dbdo = DB_DataObject::Factory('dhcp_options');
        $dbdo->whereAdd("name LIKE '$this->name'");
        if ($dbdo->find() && !isset($this->id)) {
            return _("This DHCP option already exists");
        }

        // limit the text to alphanumerics, period, and '-'
        if (!ereg("^[[:alnum:]\.\-]*$", $this->name)) {
            return _("DHCP option names are limited to alphanumerics, '.', and '-'");
        }

		return true;

	} // validateName

    // }}}
    // {{{ validateSpace()

    /**
      * Validate the space field
      *
      * @return mixed   true if valid, error string if invalid
      */
	function validateSpace() 
    {
	
        // limit the space to alphanumerics
        if (!ereg("^[[:alnum:]]*$", $this->space)) {
            return _("Values for space are limited to alphanumerics");
        }

		return true;

	} // validateSpace

    // }}}

/* Setters */
    // {{{ setEnabled()

    /**
      * Set the enabled status
      *
      * @param  mixed   true/false, 'true'/'false', 1/0
      * @return boolean true on success
      */
    function setEnabled($en)
    {
        if ($en === false || $en === 'false' || $en === 0) {
            $this->enabled = 0;
        } else {
            $this->enabled = 1;
        }

        return true;
    }

    // }}}

} // class DHCPOption

?>
