<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/dao/extended/domain.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: domain.php 4582 2006-07-06 21:03:39Z claym $

/**
  * domain.php - contains the {@link Domain} class
  *
  * @author $Author: claym $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

require_once DAODIR."/Domain.php";

/**
  * A domain object
  *
  * @package    classes
  * @subpackage dao
  */
class Domain extends DAO_Domain {
	
    // {{{ constructor

    /**
      * Constructor
      *
      * @param  int Domain ID (primary key)
      */
	function Domain ( $domain_id = 0 ) 
    {
        if ($domain_id) {
            $this->id = $domain_id;
            $this->get($this->id);
        }

	} // Domain

    // }}}

/* DBDO Overrides */
    // {{{ delete()

    /**
      * Delete this Domain and associated items
      */
    function delete($useWhere = false)
    {
		// Delete the SOA for this domain
        // "SELECT id FROM soa WHERE domain='$domain_id'"
        $dbdo = DB_DataObject::Factory('soa');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->domain = $domain_id;

        if ($dbdo->find(true)) {
            $soa = new SOA($dbdo->id);
            $soa->delete();
        }

        $dbdo = DB_DataObject::Factory('zone_domain');
        $dbdo->domain = $this->id;
        $dbdo->find();
        while($dbdo->fetch())
            $dbdo->delete();

		// Delete the object
        // "DELETE FROM domain WHERE id='$domain_id'"
        $result = parent::delete($useWhere);

		// Delete any object_history
        // "DELETE FROM domain WHERE type='domain' AND id='$domain_id'"
        /*
        $dbdo = DB_DataObject::Factory('object_history');
        $dbdo->type = 'domain';
        $dbdo->id = $domain_id;
        $dbdo->delete();
        */

	} // delete

    // }}}

/* Validators */
    // {{{ validateName()

	/**
	*
	* Checks the domain's name and ensures it is valid 
	*
	* @return	mixed   true if valid, error string if not
	*/
	function validateName()
    {
		global $conf;

        // is this name unique?


        // "SELECT id FROM domain WHERE name LIKE '$name' AND id!='$domain_id'"
        $dbdo = DB_DataObject::Factory('domain');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->whereAdd("name LIKE '$this->name'");
        if ($this->id)
            $dbdo->whereAdd("id!='$domain_id'");

        /* See if the name is being duplicated or if there is an id and we're updating */
		if ($dbdo->find() && !isset($this->id)) {
            return _("This domain name already exists in Maintain");
		}

        // validate the domain name itself
		if(!Domain::is_valid_name($this->name)) {
            return _("Invalid domain name");
		}

        return true;
	
	} //check_name

    // }}}

/* Setters */
    // {{{ setAll()
    function setAll()
    {
        $this->setEnabled($this->enabled);
    }
    // }}}
    // {{{ setEnabled()
    function setEnabled($en)
    {
        if ($en == 1 || $en == true || $en == "true") {
            $this->enabled = 1;
        } else {
            $this->enabled = 0;
        }
        return true;
    }
    // }}}

/* Getters */
    // {{{ getMaster_domain()
    function getMaster_domain()
    {
        $dom = new Domain($this->master_domain);
        return $dom->name;
    }
    // }}}

/* Static Methods */
    // {{{ is_valid_name()

	/**
	*
	* Verify a domain name is valid.
	*
	* @param	string	$name		domain name to verify	
	* @param	boolean	$allow_tld	allow $name to be a top level domain
	* @return	boolean		true if the name is valid
	* @access	public
	* @static
	*/
	function is_valid_name($name = null, $allow_tld = false) 
    {
		//TODO: http://php.oregonstate.edu/manual/en/pcre.pattern.modifiers.php
        //NOTE:(6/16/2005) These are POSIX Extended Regular Expressions
        //     refer to: http://www.php.net/manual/en/ref.regex.php
		// put newlines in these expressions.
		// PROBLEM: Requires PHP 4.3
        $regex = "^([[:alnum:]]([[:alnum:]\-]{0,61}[[:alnum:]])?\.)";
        $regex .= ($allow_tld?'+':'*')."[[:alnum:]]([[:alnum:]\-]{0,61}[[:alnum:]])?$";
        /*
        if ($allow_tld)
            $regex = "/^([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?$/";
        else
            $regex = "/^([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)*[a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?$/";
        */
        return (ereg($regex, $name));

	}

    // }}}
    // {{{ get_domain_list()

    /**
      * Retrieve a formatted domain list as an array domain arrays
      *
      * @static
      * @return array   array of all domain IDs in Maintain
      */
	function get_domain_list() 
    {
		
		/* Get the Master Domains */
		$sql = "SELECT domain.name,domain.id FROM domain LEFT JOIN domain AS second_domain ON domain.master_domain=second_domain.id WHERE second_domain.id IS NULL";
		$db_results = mysql_query($sql, dbh());
		
		while ($r = mysql_fetch_object($db_results)) { 
			$master_domains[$r->id] = $r->name;
		}

		/* Get the list of domains and their master */

        // "SELECT name,master_domain,enabled FROM domain ORDER BY domain.name ASC"
        $dbdo = DB_DataObject::Factory('domain');
        $dbdo->selectAdd();
        $dbdo->selectAdd('name,master_domain,enabled');
        $dbdo->orderBy('name ASC');
        $dbdo->find();

		while ($dbdo->fetch()) {

           /* Check for _ domains or in arpa */
           if (preg_match("/^\_/",$dbdo->name) || preg_match("/in\-addr\.arpa$/", $dbdo->name)) {
            continue;
           }
		
			if ($dbdo->enabled) { 
				$status = "Active";
			}
			else { 
				$status = "Inactive";
			}
			$data[$master_domains][$dbdo->master_domain][$dbdo->name] = $status;
		}

		return $data;	

	} // get_domain_list

    // }}}

    // {{{ _history_text()
    function _history_text()
    {
        return $this->name;
    }
    // }}}

}

?>
