<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/dao/extended/host.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// |            Zack Bartel <bartel@osuosl.org>                             |
// |            Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: host.php 4828 2006-09-18 22:02:00Z schonstal $

/**
  * host.php - contains the {@link Host} class
  *
  * @author $Author: schonstal $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

include_once DAODIR.'/Host.php';

/**
  * Host - allows for updating modifying and viewing a host
  *
  * There is a little trick to the IP stuff on our hosts now. There
  * are two fields $ip and $dynamic_range. If the host is a dynamic host
  * then IP is 0 and dynamic_range is the ranges ID of the dynamic range. If 
  * it is a static host then dynamic_range is 0 and IP is the int value of 
  * the ip address. 
  *
  * @package    classes
  * @subpackage dao
  */
class Host extends DAO_Host
{

    // {{{ vars
    /**
     * Full hardware address, : separated 
     * @var     string
     * @access  public
     */
    var $full_ha;

    /**
     * host name + domain name (constructed)
     * @var     string
     * @access  public
     */
    var $hostname;

    // }}}

    // {{{ constructor

    /**
    *
    * Constructor
    *
    * @param    int Host ID (primary key)
    */
    function Host ($host_id=0) 
    {
        if ($host_id) {
            $this->id = $host_id;
            $this->get($this->id);
        }

    }
    // }}}

    // {{{ delete()
    /**
    *
    * Deletes a host and all information connected to it
    *
    * Deletes the host from the following tables:
    * - host
    * - object_option
    * - object_history
    *
    * Although this function does not return anything it returns things
    * behind the scenes by using ui->add_delete_history
    * @return    null
    * @access    public
    */
    function delete($useWhere = false) 
    {
        global $conf,$user;

        // make sure the id is set
        if (!isset($this->id) || $this->id < 1)
            return false;
   
        // delete
        $rows = parent::delete($useWhere);

        if ($rows === false)
            return $rows;

        // Delete any options..
        // "DELETE FROM object_options WHERE type='host' AND object_id'$this->id'"
        $dbdo = DB_DataObject::Factory('object_option');
        $dbdo->whereAdd("type='host'");
        $dbdo->whereAdd("object_id='{$this->id}'");
        $dbdo->delete(DB_DATAOBJECT_WHEREADD_ONLY);

        return $rows;

    } //delete
    // }}}

// setters
    // {{{ setHa()
    function setHa($mac) 
    {
      // let's remove any invalid chr's from their mac addie
        $mac = preg_replace("/[G-Zg-z]+|\-|\.|\s+|\_|:/", "", $mac);

		// make it lowercase for the sake of apperance, and for radius..
		$this->ha = strtolower($mac);

        return true;
    }
    // }}}
    // {{{ setName()

    /**
      * Set the name (and domain if given FQDN)
      *
      * This also construct the 'hostname' field
      *
      * @param  string  hostname or FQDN
      * @return bool    true on name, hostname (, and domain) set
      */
    function setName($name) 
    {
        $hostname = split_hostname($name);

        // set the name
        $this->name = $hostname->name;

        $domain = '';

        // if it's a FQDN, set the domain too
        if ($hostname->domain) {
            $dom = DB_DataObject::Factory('domain');
            $dom->name = $hostname->domain;
            $dom->find(true);
            $this->domain = $dom->id;
            $domain = $dom->name;

        } elseif ($this->domain) {
            $dom = new Domain($this->domain);
            $domain = $dom->name;
        } else {
            $this->hostname = "$this->name";
            return false;
        }

        $this->hostname = "$this->name.$domain";

        return true;
    }
    // }}}
    // {{{ setDomain()
    function setDomain($dmn) 
    {
        if ($this->domain != $dmn) {
            // changing the domain also changes the hostname
            $this->domain = $dmn;
            $this->setName($this->name);
        }

        return true;
    }
    // }}}
    // {{{ setIp()
    function setIp($ip) 
    {
        if (!$ip) {
            $this->ip = 0;
            return true;
        }

        // this is used because IP's constructor can take either dotted quad
        // or just an int, and we can convert it safely to either type
        $this->ipobj = new IP($ip);
        $this->quad = $this->ipobj->ip_addr;
        $this->ip = $this->ipobj->ip_int;

        // unset the dynamic range
        $this->setDynamic_range(null);

        return true;
    }
    // }}}
    // {{{ setDynamic_range()
    function setDynamic_range($dr) 
    {
        if (!$dr) {
            $this->dynamic_range = 0;
            return true;
        }

        $this->dynamic_range = $dr;
        $this->setIp(null);
        return true;
    }
    // }}}
    // {{{ setExpire()
    function setExpire($ex) 
    {
        global $conf;
        if ($ex == '')
            $this->expire = '';
        else if (preg_match("/^\d{1,2}(\/)\d{1,2}(\/)\d{4}$/", $ex))
            $this->expire = strtotime($ex);
        else
            $this->expire = -1;

        return true;
    }
    // }}}
    // {{{ setWarranty()
    function setWarranty_Date($ex) 
    {
        global $conf;
        if ($ex == '')
            $this->warranty_date = '';
        else if (preg_match("/^\d{1,2}(\/)\d{1,2}(\/)\d{4}$/", $ex))
            $this->warranty_date = strtotime($ex);
        else
            $this->warranty_date = -1;

        return true;
    }
    // }}}
    // {{{ setPurch()
    function setPurchase_Date($ex) 
    {
        global $conf;
        if ($ex == '')
            $this->purchase_date = '';
        else if (preg_match("/^\d{1,2}(\/)\d{1,2}(\/)\d{4}$/", $ex))
            $this->purchase_date = strtotime($ex);
        else
            $this->purchase_date = -1;

        return true;
    }
    // }}}
    // {{{ setWorkgroup()
    /**
      * Set the workgroup for this host.
      *
      * Defaults to $this->zone's default workgroup
      * @param  int ID (primary key) of workgroup
      */
    function setWorkgroup($wkp)
    {
        if (!$wkp) {
            if (!$this->zone) 
                return false;
            $wkp = "0";
            }
            /*
            $dbdo = DB_DataObject::Factory('zone_workgroup');
            $dbdo->zone = $this->zone;
            $dbdo->is_default = 1;

            if (!$dbdo->find(true)) {
                return false;
             */
        
        $this->workgroup = $wkp;
        return true;
    }
    // }}}
    // {{{ setZone()
    /**
      * When the zone is changed, the workgroup needs to change
      * to one in the new zone
      * @param  int ID of new zone
      */
    function setZone($id)
    {
        if (!$id) {
            return false;
        }

        // if the zone has changed and we are an existing host, update the workgroup accordingly
        if (isset($this->zone) && $this->zone != $id) {
            $this->changeZone($id);
         }

        $this->zone = $id;
        return true;
    }
    // }}}
//{{{ setTtl()
    function setTtl($t)
    {
        global $conf;
        $this->ttl = $t;
        if (!$this->ttl)
            $this->ttl = $conf['soa']['default_host_ttl'];
    }
//}}}
//{{{ changeZone()
    /**
     *
     *
     *
     */
    function changeZone($id)
    {
        // get the default workgroup for this zone
        $this->zone = $id;
        $dbdo = DB_DataObject::Factory('zone_workgroup');
        $dbdo->zone = $this->zone;
        $dbdo->is_default = 1;
        $dbdo->find(true);
        $this->setWorkgroup($dbdo->workgroup);
    }
//}}}
// getters (used for displaying, for ->toArray() and possibly in forms)
    // {{{ getLast_seen()
    function getLast_seen()
    {
        if ($this->last_seen)
            return format_date($this->last_seen);
        
        return _("N/A");
    }
    // }}}
    // {{{ getIp()
    function getIp()
    {
        if ($this->ip)
            return $this->quad;
        
        return '';
    }
    // }}}
    // {{{ getDynamic_range()
    function getDynamic_range()
    {
        if (!$this->dynamic_range)
            return '';

        $range  = new Range($this->dynamic_range);
        $start  = IP::int2ip($range->start);
        $end    = IP::int2ip($range->end);

        return _("Dynamic IP Address")." ($start - $end)";
    }
    // }}}
    // {{{ getExpire()
    function getExpire()
    {
        if ($this->expire)
            //return date("F j, Y, g:i a", $this->expire);
            return date("m\/d\/Y", $this->expire);

        return _("");
    }
    // }}}
    // {{{ getWarranty()
    function getWarranty_Date()
    {
        if ($this->warranty_date)
            //return date("F j, Y, g:i a", $this->expire);
            return date("m\/d\/Y", $this->warranty_date);

        return _("");
    }
    // }}}
    // {{{ getPurch()
    function getPurchase_Date()
    {
        if ($this->purchase_date)
            //return date("F j, Y, g:i a", $this->expire);
            return date("m\/d\/Y", $this->purchase_date);

        return _("");
    }
    // }}}
    // {{{ getHa()
    function getHa()
    {
        $full_ha = '';
        $i = 0;
        while ($i < HA_LENGTH) {
            $full_ha .= $this->ha{$i++};
            if ($i%2==0 && $i != HA_LENGTH)
                $full_ha .= ':';
        }//while

        return $full_ha;
    }
    // }}}
    // {{{ getDomain()
    function getDomain()
    {
        $domain = DB_DataObject::Factory('domain');
        $domain->get($this->domain);

        return $domain->name.' ('.$this->domain.')';
    }
    // }}}
    // {{{ getZone()
    function getZone()
    {
        $zone = DB_DataObject::Factory('zone');
        $zone->get($this->zone);

        return $zone->name . " ($this->zone)";
    }
    // }}}
    // {{{ getWorkgroup()
    function getWorkgroup()
    {
        $workgroup = DB_DataObject::Factory('workgroup');
        $workgroup->get($this->workgroup);
        return $workgroup->name;
    }
    // }}}
    // {{{ getHostname()
    function getHostname()
    {
        if (!$this->hostname) {
            $this->setName($this->name);
        }

        return $this->hostname;
    }
    // }}}

// parent class (Maintain.php) implementations
    // {{{ setAll()
    function setAll() 
    {
        $this->setName($this->name);
        $this->setDomain($this->domain);
        $this->setHa($this->ha);
        $this->setIp($this->ip);
        $this->setDynamic_range($this->dynamic_range);
//        $this->setZone($this->zone);
        $this->setWorkgroup($this->workgroup);
    }
    // }}}
    // {{{ _toArray()
    function _toArray()
    {
        $arr = array();

        $arr['hostname'] = $this->hostname;

        return $arr;
    }
    
    // }}}
    // {{{ _history_text()
    function _history_text()
    {
        // : HOSTNAME (FULL_HA) 'IP': IP
        return $this->getHostname() . '( ' . $this->getHa() . ' )' .
            _("IP") . ': ' . $this->getIp();
    }
    // }}}

// other functions
    // {{{ has_host_history()

    /**
    *
    * Checks to see if host has a history
    *
    * Uses the host_history table and the hardware address
    *
    * @return   boolean True if there is host history, PEAR_Error if HA not set
    * @access    public
    */
    function has_host_history() 
    {
        global $conf;

        if (!isset($this->ha)) {
            return new PEAR_Error(
                    _("Cannot check for host history without setting a hardware address first"));
        }

        // "SELECT id FROM host_history WHERE ha='$this->ha' LIMIT 1"
        $dbdo = DB_DataObject::Factory('host_history');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->ha = $this->ha;

        if ($dbdo->find(true))
            return true;

        return false;

    } //has_host_history

    // }}}
    // {{{ is_active()

    /**
    *
    * Check to see if host is active
    *
    * Uses $conf['general']['update_time'] and compares it to the current time.  If
    * The host has been seen since we last update then it is assumed 
    * active.
    *
    * @return    boolean    true if the host has been seen, PEAR_Error if no id is set yet
    * @access    public
    */
    function is_active() 
    {
        global $conf;

        if (!isset($this->id)) {
            return new PEAR_Error(
                    _("Cannot determine if host is active without retrieving DB info first"));
        }

        $history = $this->get_history();

        if ($history->date + $conf['general']['update_time'] <= time()) 
            return true;

        return false;

    } //is_active

    // }}}
    // {{{ get_host_history()

    /**
      * Return a list of Host_History IDs for this instantiated Host
      *
      * @return array   host_history IDs (primary key) for this host
      */
    function get_host_history() 
    {
        global $conf;

        $dbdo = DB_DataObject::Factory('host_history');
        $dbdo->selectAdd();
        $dbdo->selectAdd("DISTINCT(date) as date,ip,ha");

        // Check to see if we are using a real object
        // This is bassed of $this->ha
        if ($this->ha) {

            // "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ha='$this->ha' ORDER BY date DESC"
            $dbdo->ha = $this->ha;

        } elseif ($this->ip) {

            // "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ip='$this->ip' ORDER BY date DESC"
            $dbdo->ip = $this->ip;

        } elseif ($ip_int) {

            //If they passed a ip_int base it on that;
            // "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ip='$ip_int' ORDER BY date DESC"
            $dbdo->ip = $ip_int;

        } else {

            // this host is not populated with data, so it really doesn't exist
            return array();

        }

        $dbdo->orderBy('date DESC');
        $dbdo->find();

        while ($dbdo->fetch()) {
            $results[] = $dbdo->id;
        } //end while
    
        return $results;

    } // get_host_history

    // }}}

// static functions
// {{{ bulk_get()
    /**
    *
    * Gets (and creates) an array of host IDs based on the parameters
    *
    *
    * @deprecated     This should be deprecated soon and put into a seperate
    *        wrapper class
    * @access    public
    * @param     $select_type    Selects the type of search.
    *                - Defined Range - range ID @see Range 
    *                - Custom Range - by start and end ranges
    *                - Domain - via domain id @see Domain
    *                - Hostname - via human readable hostname
    *                - Other - includes all available host 
    *                  fields and a $value
    * @param    $range_id    For "Defined Range" @see Range
    * @param    $start_range    For "Custom Range" - start of IP range search
    * @param    $end_range    For "Custom Range" - end of IP range search
    * @param    $domain_id    For "Domain" - @see Domain
    * @param     $hostname    For "Hostname" 
    * @param    $select_field    For custom search
    * @param    $value        For custom search - the value of the 
    *                field
    * @return   array   array of affected hosts (ids only) or PEAR_Error on failure
    */
    function bulk_get($select_type, $range_id=0, $start_range=0, $end_range=0,
              $domain_id=0, $hostname=0, $select_field=0, $value=0) 
    {
        // TODO we should really throw a pear::error on failure
        global $conf;

        // ok we've got to get the hosts for viewing
        // check to see what kind of select they did and grab the
        // correct hosts

        // zone -> range grab
        if ($select_type == "defined_range") {
                $range = new Range($range_id);
                $hosts = $range->get_range_hosts();
        }

        // custom range grab
        elseif ($select_type == "custom_range") {
            // check to make sure range was entered correctly
            if (! Net_IPv4::validateIP($start_range))
                return new PEAR_Error(_("Invalid start range"));

            if (! Net_IPv4::validateIP($end_range)) 
                return new PEAR_Error(_("Inalid end range"));

            $start_int = Net_IPv4::ip2double($start_range);
            $end_int = Net_IPv4::ip2double($end_range);

            if ($start_int >= $end_int)
                return new PEAR_Error(_("End range must be larger than start range"));

            $host = new Host();
            $host->selectAdd();
            $host->selectAdd('id');
            $host->whereAdd("ip <= '$end_int'");
            $host->whereAdd("ip >= '$start_int'");
            $host->whereAdd("zone = '{$_SESSION['zone']}'");
            $host->orderBy("ip DESC");

            $count = $host->find();
            $hosts = array();
            while($host->fetch())
                $hosts[] = $host->id;

        } // custom range grab

        // zone -> domain grab
        elseif ($select_type == "domain") {
                $domain = new Domain($domain_id);
                $hosts = $domain->get_hosts();
        }
        // fuzzy host grab
        elseif ($select_type == "hostname") {
                if (substr($hostname,strlen($hostname)-1,1) != "%") { $hostname = $hostname ."%"; }
                if (substr($hostname,0,1) != "%") { $hostname = "%" . $hostname; }

                $host = new Host();
                $host->selectAdd();
                $host->selectAdd('id');
                $host->whereAdd("name LIKE '$hostname'");
                $host->setZone($_SESSION['zone']);

                $host->find();
                $hosts = array();
                while ($host->fetch())
                    $hosts[] = $host->id;
        }

        elseif ($select_type == 'other') {
            $value = "%" . $value . "%";
            $host = new Host();
            $host->selectAdd();
            $host->selectAdd('id');
            $host->whereAdd("$select_field LIKE '$value'");
            $host->setZone($_SESSION['zone']);

            $host->find();
            $hosts = array();
            while ($host->fetch())
                $hosts[] = $host->id;
        }
        
        return $hosts;

    } // bulk_get
    // }}}

// validation functions
    // {{{ validateWorkgroup()
    /**
     * Checks to make sure a workgroup can be added to the specified zone.
     */
    function validateWorkgroup() 
    {
        global $conf, $user;

        // admins can assign any workgroup apparently
        /* I feel like this should be in the form itself,
         * and not in the DAO
         */
        /*
        if ($user->is_admin()) { return true; }
        */

        // make sure a zone was set
        if (!$this->zone || $this->zone == "" || $this->zone == 0)
            return _("Attempting to validate a host's workgroup without setting a zone first");

        if ($this->workgroup == "")
            return true;
 
        // make sure this workgroup is part of the controlling zone
        $wobj = new Workgroup($this->workgroup);
        if (!$wobj->has_zone($this->zone))
            return _("The workgroup is not part of the controlling zone.");

        // otherwise, return true
        return true;
    }
    // }}}
    // {{{ validateName()
    function validateName() 
    {
        global $conf;

        // make sure a name has been entered
        if (!strlen($this->name))
            return _("No name has been entered for this host.");

        // because we format our dynamic hosts 128-193-188-118.yoursite.com we can't let people
        // create hosts like that normaly
        if ( Net_IPv4::validateIP($this->name) )
            return "$this->name "._("is an invalid hostname as it would overlap with auto-generated dynamic hosts");

        // make sure it's a valid hostname
        if (!preg_match("/^[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9]$/", $this->name) )
            return "$this->hostname "._("is an invalid hostname");

        // we have to use the host_id to determine if they working on that host currently
        //   this is used during the update of a single host
        // we do a like here in so we get the case-insensitive match

        // "SELECT * FROM host WHERE name LIKE '$name' AND domain='$this->domain'"

        // make sure a domain is set
        if (!isset($this->domain))
            return _("Cannot validate hostname without specifying a domain first");
       
        // make sure a hostname is set
        if (!isset($this->name))
            return _("Hostnames cannot be left blank.");
 
        // make sure this FQDN doesn't exist
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->domain = $this->domain;
        $dbdo->whereAdd("name LIKE '{$this->name}'");
        if ($this->id)
            $dbdo->whereAdd("id!='{$this->id}");

        if ($dbdo->find()) 
            return "$this->hostname "._("is already in use by a host.");


        /* make sure there isn't a forward pointer with this name */

        // "SELECT id FROM pointer WHERE hostname LIKE '"$this->hostname"'"
        $dbdo = DB_DataObject::Factory('pointer');
        if (!isset($this->hostname))
            $this->setName($this->name);

        $dbdo->whereAdd("hostname LIKE '{$this->hostname}'");
    
        if ($dbdo->find())
            return "$this->hostname "._("is already in use by a Pointer");

        /* ensure there isn't a Cname with this name */

        // "SELECT id FROM cname WHERE name LIKE '$name' AND domain='$this->domain'"
        $dbdo = DB_DataObject::Factory('cname');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->domain = $this->domain;
        $dbdo->whereAdd("name LIKE '{$this->name}'");
        
        if ($dbdo->find())
            return "$this->hostname "._("is already in use by a CNAME.");

        // if we made it here, it's valid
        return true;
    }
    // }}}
    // {{{ validateHa()
    function validateHa() 
    {
        global $conf;

        // make sure it's 12 hex-digits long
        if (strlen($this->ha) != HA_LENGTH) {
            $msg =  _("Hardware address must be exactly");
            $msg .= ' '.HA_LENGTH.' '._("characters long, or").' ';
            $msg .= (HA_LENGTH+5).' '._("with colons");
            return $msg;
        }

        // this is a special case for people who don't want to
        //   be in DHCP ... some ranges don't want to be there.
        // This is sort of kludgy but given the extremely special cases
        //  that it is used in it will have to remain, for now.
        // This option can now be set by Zone admins on a per zone basis so that
        //  we can control some of the rampant abuse of this feature.
        $zone = new Zone($this->zone);
        if (!strcmp($this->ha, '000000000000') && !$zone->allow_blank_ha) {
            $msg = _("Hardware address cannot be all zeroes.").'  ';
            $msg .= _("This would allow you to to bypass DHCP.").'  ';
            $msg .= _("Your admin has disabled this option.");
            return $msg;
        }

        // does it have invalid chars
        // note: find a locale-sensitive way to test for hex characters
        if (preg_match("/[^0-9a-fA-F]+/", $this->ha))
            return _("Hardware addresses can only contain hexidecimal numbers (0-9A-F)");

        //check for duplicate ha in same zone
        if ($this->dynamic_range)
            $sql = "SELECT host.id FROM host WHERE ha='{$this->ha}' AND dynamic_range='{$this->dynamic_range}'";
        else {
            // if there isn't a dynamic range set up, we need to figure out
            // the encapsulating range to determine the subnet.
            $dbdo = DB_DataObject::Factory('ranges');
            $dbdo->whereAdd("start<='{$this->ip}'");
            $dbdo->whereAdd("end>='{$this->ip}'");
            $result = $dbdo->find(true);
            $subnet = 0;
            if ($result) {
                $range = new Range($dbdo->id);
                $subnet = $range->subnet;
            }
            $sql = "SELECT host.id FROM host,ranges WHERE ranges.subnet='".$subnet."' AND host.ip<=ranges.end AND host.ip>=ranges.start  AND host.ha='".$this->ha."' ";
        }  // if range
        
        // this is for the case when we are actually updating a host
        if ( $this->id)
            $sql .= "AND host.id!='{$this->id}'";

        $db_result = mysql_query($sql, dbh());
        $host_info = mysql_fetch_object($db_result);

        // Ensure there are no duplicates but allow all zeros
        if ($host_info && ($this->ha != '000000000000')) {
            $host = new Host($host_info->id);
            return _("The hardware address")." $this->ha "._("is already in use by")." $host->hostname";
        }

        // if we're here, we're valid
        return true;
    }
    // }}}
    // {{{ validateTtl()
    function validateTtl() 
    {
        global $conf;

        // blah;
        // this REALLY belongs as part of the form!

        if (!is_numeric($this->ttl))
            return _("TTL must be numeric");

        return true;

    }
    // }}}
    // {{{ validateDomain()
    function validateDomain() {
        global $conf, $user;

        // has the domain been set?
        if (!isset($this->domain))
            return _("No domain was specified.");

        // is this zone part of this domain?
        $dbdo = DB_DataObject::Factory('zone_domain');
        $dbdo->zone = $this->zone;
        $dbdo->domain = $this->domain;

        // if not, return the error string
        if (!$dbdo->find()) {
            // for the domain name:
            $domain = new Domain($this->domain);
            if(!$user->is_admin())
                return _("You lack the necessary permissions to put a host in the domain").
                    " $domain->name";
        }

        return true;

    }
    // }}}
    // {{{ validateExpire()
    function validateExpire() {
        if ($this->expire == '')
            return true;

        if ($this->expire == -1)
            return _("Invalid expire value.  Example: 03/27/1984 (MM/DD/YYYY)");
        
        if (time() >= $this->expire)
            return _("Expire date must be in the future");

        return true;
    }
    // }}}
    // {{{ validateWarranty()
    function validateWarranty_Date() {
        if ($this->warranty_date == '')
            return true;

        if ($this->warranty_date == -1)
            return _("Invalid warranty date.  Example: 03/27/1984 (MM/DD/YYYY)");
        
        return true;
    }
    // }}}
    // {{{ validatePurch()
    function validatePurchase_date() {
        if ($this->purchase_date == '')
            return true;

        if ($this->purchase_date == -1)
            return _("Invalid pruchase date.  Example: 03/27/1984 (MM/DD/YYYY)");
        
        return true;
    }
    // }}}

    // {{{ validateIp()
    function validateIp() 
    {
        global $conf, $user;

        // make sure only an IP or Range has been set, but not both
        if ($this->ip && $this->dynamic_range) {
            $msg = _("You have defined both an IP address and an IP range.");
            $msg .= _("Please choose one or the other.");
            return $msg;
        }

        // if there's only a dynamic_range, we're ok (potentially)
        elseif ($this->dynamic_range)
            return true;

        // prep an IP object
        //$ip = new IP($this->ip);

        // make sure it's really an IP address
       /* if($this->ipobj->origin == "other") { //convert it if it's an int
            $ip_int = IP::int2ip($this->ip);
        } else {
            $ip_int = $this->quad;
        }
        */

        //this will fail out if the IP was entered as an int, or as an invalid string.
        if($this->ipobj->origin == "other")
            return _("IP must be a valid dotted quad (x.x.x.x)");
        
        //Make sure it's a valid IP
        //if (!Net_IPv4::validateIP($this->quad)) {
        //    return _("Invalid IP address")." $ip_int";
        //}
        
        // make sure the IP isn't in use
        // "$sql = "SELECT host.id,host.name FROM host WHERE ip='$ip_int'";"
        $dbdo = new Host();
        $dbdo->selectAdd();
        $dbdo->selectAdd('id, name');
        $dbdo->whereAdd("ip='$this->ip'");
        if ($this->id)
            $dbdo->whereAdd("id!='$this->id'");

        //check to see if it's already in use by a host (that's not the current host on edit)
        if ($dbdo->find(true) /*&& !isset($this->id)*/)
            return _("IP address already in use by host")." $dbdo->hostname";

        //now check to make sure it isn't in a dynamic range
        // $sql = "SELECT id FROM ranges WHERE start<='$ip_int' AND end>='$ip_int'";
        $dbdo = DB_DataObject::Factory('ranges');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->whereAdd("start<='{$this->ip}'");
        $dbdo->whereAdd("end>='{$this->ip}'");
        $dbdo->find(true);

        if ($dbdo->type == "dynamic" or $dbdo->type == "ddns")
            return _("You cannot add a specific IP in a dynamic range.");

        $zr = DB_DataObject::Factory('zone_range');
        $zr->selectAdd();
        $zr->selectAdd("zone");
        $zr->whereAdd("range = $dbdo->id");
        $zr->find(true);

        if ($zr->zone != $_SESSION['zone'])
            return _("The specified IP is not in the current zone");

        /* blah;
         * this is broken until conf['zone'] is set up...but I like sessions better
         * I still wonder if this doesn't belong in the form itself, 
         * and not in the DAO
         */
        /*
        $dbdo = new Range($dbdo->id);
        if (!$dbdo->has_zone($conf['zone']->id) && !$user->is_admin()) {
            $error = "The specified IP is not in the current Zone";
            $conf['error']->add_error('specified_ip', $error);
            $conf['error']->add_error('general', $error);
            return false;
        }
        */

        return true;
    }
    // }}}
    // {{{ validateDynamic_range()
    function validateDynamic_range() {
        global $conf;

        // make sure only an IP or Range has been set, but not both
        if ($this->ip && $this->dynamic_range) {
            $msg = _("You have defined both an IP address and an IP range.");
            $msg .= _("Please choose one or the other.");
            return $msg;
        }

        // if there's only an IP, we're ok (potentially)
        elseif ($this->ip)
            return true;

        $range = new Range($this->dynamic_range);
        if (!$range->count_free_addresses())
            return _("There are no free IP addresses in this range.");

        //validate overflow
        //there can only be adresses * conf[dynamic_overflow_multiple]
        //TODO: MNT-1482
        $mult = $conf['general']['dynamic_overflow_multiple'];
        
        $range = new Range($this->dynamic_range); 
        $range_hosts = $range->count_range_hosts();
        $free_addr = abs($range->count_free_addresses());
        $capacity = $range->end - $range->start;

        if ($free_addr < 0) //Overfilled
        {
            $free_addr = abs($free_addr);
            if ($free_addr >= $capacity * $mult)
                return _("Exceeded maximum overfilled hosts");
        }

        return true;

    }
    //}}}
//{{{ validateEnabled()

    function validateEnabled()
    {
        //Attempting to enable
        if($this->enabled == 1) {
            if($this->ip == 0 && $this->dynamic_range == 0)
                return _("A host must have either an IP address or be part of a dynamic range to be enabled.");
        }

        return true;
    }

//}}}

} // class Host

?>
