<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: MaintainForm.php 4815 2006-09-14 18:01:29Z wenzel $

/**
 *  MaintainForm.php - contains the {@link MaintainForm} class
 *
 *  @author $Author:$
 *  @filesource
 *  @package    htdocs
 *  @subpackage forms
 */

define("FORM", 1);
define("DBDO", 2); 

$phphtmllib = PHPHTMLLIB; 
require_once($phphtmllib."/includes.inc");
require_once($phphtmllib."/form/includes.inc");

/**
 *  This class is an extension of FormContent to provide extra functionality fo
 *  the various forms found throughout Maintain.
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class MaintainForm extends FormContent
{

/*Vars*/
//{{{   vars

    /**
     *  This is is to be overloaded by the child class and is used to remap values
     *  from the form to the db dataobjects and vice versa.
     */
    var $map = array();

    /**
     *  Where cancel goes if we don't change its behavior
     */
    var $cancel_loc;
    
    /**
     *  Cancel loc's get vars.  Must be an array to be useful
     */
    var $cancel_get;
    
//}}}

/*Methods*/
//{{{   remap()

    /**
     * Map (forwards and reverse) array indexes between the form and
     * the represented DB_DataObject.
     *
     * Array indexes are set to the form labels by default, and in order
     * to do a ->setFrom() on a db_dataobject, we need the indexes to be
     * the database column names.  This function returns the array of form
     * elements indexed accordingly
     *
     *   @param  array   the array to re-index
     *   @param  int     output format constant: DBDO or FORM
     * @return array   copy of the original array, but with new indexes
                       or an empty array if no internal $map variable was present
     */
    function remap($array, $output = FORM)
    {
        // setup the return value
        $result = array();

        // loop through the array
        foreach($this->map as $form=>$dbdo) {

            // if going from DBDO to LABEL
            if ($output == FORM) {
                $form = ereg_replace("_"," ",$form);
                $result[$form] = $array[$dbdo];
            } else {
                $result[$dbdo]  = $array[$form];
            }
        }

        return $result;
    }

//}}}
//{{{   add_elements()

    /**
     *  This function is meant to allow the user to add as many elements at a time
     *  as they want.
     *
     *  @param Mixed    The elements that are to be added to the list
     */
    function add_elements()
    {
        $elements = func_get_args();
        $count = func_num_args();
      
        //loop through the items.  Don't ask me why but foreach chokes...
        for($i = 0; $i < $count; $i++) {
            $this->add_element($elements[$i]); //add them
        }

    }

//}}}
//{{{   contains_element()

    /**
     *  This method determines if there is an element in the form with the given
     *  label.  It is essentially a wrapper for array_key_exists().
     *
     *  @param  String  $label  The label to be checked for
     *
     *  @return Boolean         True if form has element with given label
     */
    function contains_element($label)
    {
        return array_key_exists($label, $this->_elements);
    }

//}}}
//{{{   form()

    /**
     *  This method renders the form to the screen
     *
     *  @return DIVTag  $div    Div containing the form
     */
    function form()
    {
        $div = html_div("form");
        $table = html_table();

        //alternating table row classes
        $class = array(0=>"even",
                        1=>"odd");

        $temp = 1;

        //Add the form elements
        foreach($this->_elements as $label=>$element) {
            $tr = html_tr($class[++$temp%2]);
            $tr->add($this->element_label($label),$this->element_form($label));
            $table->add($tr);
        }
        
        $table->add($this->form_buttons($class[++$temp%2])); //add the buttons

        $div->add($table);

        $div->add($this->form_hidden()); //add any hidden variables

        return $div;
    }

//}}}
//{{{   form_buttons()

    /**
     *  This method should be overloaded by the child class to contain the buttons
     *  that are to appear below the form.
     *
     *  @param  String  $class  The class for the table row of buttons
     *
     *  @return TRTag   $tr     A table row with buttons
     */
    function form_buttons($class = null)
    {
        $tr = html_tr($class);
        if(!isset($this->cancel_loc))
            $this->set_cancel_loc($_SERVER['SCRIPT_NAME']);
        if(!isset($this->cancel_get))
            $this->set_cancel_get(array());
        $this->set_cancel_action($this->_build_cancel_link($this->cancel_loc, $this->cancel_get));
        $tr->add("&nbsp;",container($this->add_action(_("Submit")),$this->add_cancel()));
        return $tr;
    }

//}}}
//{{{   form_hidden()

    /**
     *  This method should be overloaded by the child class and is used to add the
     *  hidden form variables to the form.
     *
     *  @return FEHidden    A hidden form element
     */
    function form_hidden()
    {
        $hidden = new FEHidden("action",$_REQUEST['action']);
        $this->add_element($hidden);
        return $this->element_form(_("action"));
    }

//}}}
//{{{   _build_cancel_link()

    /**
     *  This method builds a link for the cancel button on the bottom of the form.
     *
     *  @param  String  $page   The URL
     *  @param  Array   $get    An array of GET vars
     */
    function _build_cancel_link($page, $get = array())
    {
        $link = $page;

        if(is_array($get) && !empty($get)) {
            $lim = count($get);
            $i = 0;
            $link .= "?";
            foreach($get as $id=>$val) {
                if($i < $lim && !$i)
                    $link .= "&amp;";
                $link .= "$id=$val";
                $i++;
            }
        }

        return $link;
    }

//}}}
//{{{ set_cancel_loc()

    /**
     *  Setter method
     *
     *  @param  String  $loc    URL
     */
    function set_cancel_loc($loc)
    {
        $this->cancel_loc = $loc;
    }

//}}}
//{{{ set_cancel_get()

    /**
     *  Setter method
     *
     *  @param  Array   $get    Array of GET vars
     */
    function set_cancel_get($get)
    {
        $this->cancel_get = $get;
    }

//}}}

}

?>
