<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel <bartel@osuosl.org>                             |
// |                                                                        |
// +------------------------------------------------------------------------+
// $Id: UserDeleteConfirm.php 4283 2006-03-21 00:13:52Z wenzel $

/**
 *  UserDeleteConfirm.php - contains the {@link UserDeleteConfirm} class
 *
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */


/**
 *  Simple confirmation page for an admin when fully deleting a user 
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class UserDeleteConfirm extends FormContent
{

//{{{   form_init_elements()

    /**
     *  This method is where all the form elements get intialized before use
     */
    function form_init_elements()
    {
        $hidden = new FEHidden("action",$_REQUEST['action']);
        $this->add_element($hidden);
        $u = new FEHidden("username",$_REQUEST['username']);
        $this->add_element($u);
    }

//}}}
//{{{   form()

    /**
     *  This method lays out the form to the user and also adds the the actions
     *  to the page.
     *
     *  @return DIVTag  $div    Div with the form inside
     */
    function form()
    {
        $u = $_REQUEST['username'];
        $this->set_cancel_action($_SERVER["SCRIPT_NAME"]."?action=show&amp;type=local");
        
        $div = html_div("form");
        $div->add(_("WARNING! You are about to completely remove the user: ") . $u . _(" from Maintain. Are you sure?"), html_br(2)); 
        $div->add($this->add_action(_("Confirm")),$this->add_cancel());
        $div->add($this->element_form(_("action")));

        $div->add($this->form_hidden());
        return $div;
    }

//}}}
//{{{   form_action()

    /**
     *  This is the method that performs the desired action.  In this case it
     *  deletes the user from Maintain if the action was confirmed.
     *
     *  @return Boolean     True if the purge was successful
     */
    function form_action()
    {
        $res = true;
    
        switch(strtolower($this->get_action())) {

            case 'confirm':
                $usr = new User($_REQUEST['username']);
                $usr->query("BEGIN");
                $res = $usr->delete();

                if ($res == false){
                    $usr->query("ROLLBACK");
                    $this->add_error(NULL, _("An error occured while deleting the user"));
                }
                
                else{
                    $usr->query("COMMIT");
                    $usr->write_history(_("User Deleted"), ACTION_DELETE, $_REQUEST['username'], $_SESSION['zone']);
                }
                break;
                
            default:
                die_hard(_("Invalid form action"));
                break;
        }


        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method displays a message indication to the user that their action
     *  was complete successfully.
     *
     *  @return DIVTag  $div    Div that contains the success message
     */
    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a(web_root()."index.php", _("[Go to Index]"));
        $userlink = html_a(web_root()."user.php?action=show&amp;type=local",_("[Go to User List]"));
        $msg = _("User").' '.$_REQUEST['username'].' '._("was successfully deleted from Maintain");
        $div->add($msg,html_br(),$indlink,$userlink);

        return $div;
    }
//}}}
//{{{   form_backend_validation()
    /*  Make sure that it is ok to delete this user
     *
     *  @return true if valid to delete user false otherwise
     */
    function form_backend_validation()
    {
        global $user;
        
        $res = true;
        $username = $_REQUEST['username'];
        if ($username == NULL || $username == '')   {
            $this->add_error(NULL, _("Invalid username specified"));
            $res = false; 
        }

        /* Can't delete self */
        if (!strcasecmp($user->username, $username)) {
            $this->add_error(NULL, _("Deletion of own user is not allowed"));
            $res = false;
        } 
        
        return $res; 
    }
    
//}}}
//{{{ form_hidden()
/** 
 *  Hide username in a hidden form value to propogate it through the request
 *
 *  @return DIVTag  $div    Div that contains
 */
    function form_hidden()
    {
        $hidden = new FEHidden('username', $_REQUEST['username']);
        $this->add_element($hidden);
        return $this->element_form('username');
    }
//}}}

}

?>
